@@ -297,6 +297,23 @@ setClass("H2OWordEmbeddingModel", contains="H2OModel")
 #' @export
 setClass("H2OCoxPHModel", contains="H2OModel")
 
+#' @rdname H2OCoxPHModel-class
+#' @param object an \code{H2OCoxPHModel} object.
+#' @export
+setMethod("show", "H2OCoxPHModel", function(object) {
+  requireNamespace("survival")
+  o <- object
+  model.parts <- .model.parts(o)
+  m <- model.parts$m
+  cat("Model Details:\n")
+  cat("==============\n\n")
+  cat(class(o), ": ", o@algorithm, "\n", sep = "")
+  cat("Model ID: ", o@model_id, "\n")
+
+  # summary
+  get("print.coxph", getNamespace("survival"))(.as.survival.coxph.model(o@model))
+})
+
 #'
 #' The H2OCoxPHModelSummary object.
 #'
@@ -307,37 +324,21 @@ setClass("H2OCoxPHModel", contains="H2OModel")
 #' @export
 setClass("H2OCoxPHModelSummary", representation(summary="list"))
 
-#' @rdname H2OCoxPHModel-class
-#' @param object an \code{H2OCoxPHModel} object.
+#' @rdname H2OCoxPHModelSummary-class
+#' @param object An \code{H2OCoxPHModelSummary} object.
 #' @export
-setMethod("show", "H2OCoxPHModel", function(object) {
-    o <- object
-    model.parts <- .model.parts(o)
-    m <- model.parts$m
-    cat("Model Details:\n")
-    cat("==============\n\n")
-    cat(class(o), ": ", o@algorithm, "\n", sep = "")
-    cat("Model ID: ", o@model_id, "\n")
-
-    # summary
-    get("print.coxph", getNamespace("survival"))(.as.survival.coxph.model(o@model))
+setMethod("show", "H2OCoxPHModelSummary", function(object) {
+  requireNamespace("survival")
+  get("print.summary.coxph", getNamespace("survival"))(object@summary)
 })
 
 #'
-#' Print the CoxPH Model Summary
-#'
-#' @param object An \linkS4class{H2OCoxPHModelSummary} object.
-#' @param ... further arguments to be passed on (currently unimplemented)
-#' @export
-setMethod("show", "H2OCoxPHModelSummary", function(object)
-  get("print.summary.coxph", getNamespace("survival"))(object@summary))
-
-#'
-#' Print the CoxPH Model Summary
+#' Summary method for H2OCoxPHModel objects
 #'
 #' @param object an \code{H2OCoxPHModel} object.
 #' @param conf.int a specification of the confidence interval.
 #' @param scale a scale.
+#' @importFrom stats qnorm
 #' @export
 setMethod("summary", "H2OCoxPHModel",
           function(object, conf.int = 0.95, scale = 1) {
@@ -345,7 +346,7 @@ setMethod("summary", "H2OCoxPHModel",
             if (conf.int == 0)
               res@summary$conf.int <- NULL
             else {
-              z <- stats::qnorm((1 + conf.int)/2, 0, 1)
+              z <- qnorm((1 + conf.int)/2, 0, 1)
               coef <- scale * res@summary$coefficients[,    "coef",  drop = TRUE]
               se   <- scale * res@summary$coefficients[, "se(coef)", drop = TRUE]
               shift <- z * se
@@ -360,9 +361,20 @@ setMethod("summary", "H2OCoxPHModel",
             res
           })
 
+#' @rdname H2OCoxPHModel-class
+#' @param ... additional arguments to pass on.
+#' @export
 coef.H2OCoxPHModel        <- function(object, ...) .as.survival.coxph.model(object@model)$coefficients
+
+#' @rdname H2OCoxPHModelSummary-class
+#' @param ... additional arguments to pass on.
+#' @export
 coef.H2OCoxPHModelSummary <- function(object, ...) object@summary$coefficients
 
+#' @rdname H2OCoxPHModel-class
+#' @param fit an \code{H2OCoxPHModel} object.
+#' @param scale optional numeric specifying the scale parameter of the model.
+#' @param k numeric specifying the weight of the equivalent degrees of freedom.
 extractAIC.H2OCoxPHModel <- function(fit, scale, k = 2, ...) {
   fun <- get("extractAIC.coxph", getNamespace("stats"))
   if (missing(scale))
@@ -371,11 +383,69 @@ extractAIC.H2OCoxPHModel <- function(fit, scale, k = 2, ...) {
     fun(.as.survival.coxph.model(fit@model), scale = scale, k = k)
 }
 
-logLik.H2OCoxPHModel <- function(object, ...)
+#' @rdname H2OCoxPHModel-class
+#' @export
+logLik.H2OCoxPHModel <- function(object, ...) {
+  requireNamespace("survival")
   get("logLik.coxph", getNamespace("survival"))(.as.survival.coxph.model(object@model), ...)
+}
 
-vcov.H2OCoxPHModel <- function(object, ...)
+#' @rdname H2OCoxPHModel-class
+#' @param formula an \code{H2OCoxPHModel} object.
+#' @param newdata an optional \code{H2OFrame} or \code{data.frame} with the same
+#' variable names as those that appear in the \code{H2OCoxPHModel} object.
+#' @importFrom stats as.formula
+#' @export survfit.H2OCoxPHModel
+survfit.H2OCoxPHModel <-
+function(formula, newdata, ...)
+{
+  requireNamespace("survival")
+  if (missing(newdata)) {
+    if (!is.null(formula@allparameters$stratify_by) ||
+        !is.null(formula@allparameters$interactions) ||
+        !is.null(formula@allparameters$interaction_pairs)) {
+      stop("Models with strata or interaction terms require newdata argument")
+    }
+    newdata <- as.data.frame(c(as.list(as.data.frame(formula@model$x_mean_cat)),
+                               as.list(as.data.frame(formula@model$x_mean_num)),
+                               as.list(as.data.frame(formula@model$mean_offset))),
+                             col.names = c(formula@model$coefficients_table$names,
+                                           formula@model$offset_names))
+  }
+  if (is.data.frame(newdata))
+    capture.output(newdata <- as.h2o(newdata))
+
+  # Code below has calculation performed in R
+  capture.output(suppressWarnings(pred <- as.data.frame(h2o.predict(formula, newdata))[[1L]]))
+  res <- list(n         = formula@model$n,
+              time      = formula@model$time,
+              n.risk    = formula@model$n_risk,
+              n.event   = formula@model$n_event,
+              n.censor  = formula@model$n_censor,
+              surv      = NULL,
+              type      = ifelse(length(as.formula(formula@model$formula)[[2L]]) == 3L, "right", "counting"),
+              cumhaz    = formula@model$cumhaz_0,
+              std.err   = NULL,
+              upper     = NULL,
+              lower     = NULL,
+              conf.type = NULL,
+              conf.int  = NULL,
+              call      = match.call())
+  if (length(pred) == 1L)
+    res$cumhaz <- res$cumhaz * exp(pred)
+  else
+    res$cumhaz <- outer(res$cumhaz, exp(pred), FUN = "*")
+  res$surv <- exp(- res$cumhaz)
+  oldClass(res) <- c("survfit.H2OCoxPHModel", "survfit.cox", "survfit")
+  res
+}
+
+#' @rdname H2OCoxPHModel-class
+#' @export
+vcov.H2OCoxPHModel <- function(object, ...) {
+  requireNamespace("survival")
   get("vcov.coxph", getNamespace("survival"))(.as.survival.coxph.model(object@model), ...)
+}
 
 #'
 #' Accessor Methods for H2OModel Object
@@ -733,7 +803,7 @@ setMethod("summary", "H2OGrid",
 #' This class represents an H2OFrame object
 #'
 #' @export
-setClass("H2OFrame")
+setClass("H2OFrame", contains = "environment")
 
 #'
 #' The H2OAutoML class
