@@ -190,8 +190,6 @@ h2o.init <- function(ip = "localhost", port = 54321, startH2O = TRUE, forceDL =
         print(rv$httpStatusCode)
         print(rv$curlErrorMessage)
 
-        #try a hail mary curl
-        print(system("curl 'http://localhost:54321'"))
         stop("H2O failed to start, stopping execution.")
       }
     } else
@@ -497,6 +495,21 @@ h2o.clusterStatus <- function() {
 
 .Last <- function() { if ( .isConnected() ) try(.h2o.__remoteSend("InitID", method = "DELETE"), TRUE)}
 
+#
+# Returns error string if the check finds a problem with version.
+# This implementation is supposed to blacklist known unsupported versions.
+#
+.h2o.check_java_version <- function(jver = NULL) {
+  if(any(grepl("GNU libgcj", jver))) {
+    return("Sorry, GNU Java is not supported for H2O.")
+  }
+  # NOTE for developers: keep the following blacklist in logically consistent with whitelist in java code - see water.H2O.checkUnsupportedJava, near line 1849
+  if (any(grepl("^java version \"1\\.[1-6]\\.", jver))) {
+    return(paste0("Your java is not supported: ", jver[1]))
+  }
+  return(NULL)
+}
+
 .h2o.startJar <- function(ip = "localhost", port = 54321, nthreads = -1, max_memory = NULL, min_memory = NULL,
                           enable_assertions = TRUE, forceDL = FALSE, license = NULL, extra_classpath = NULL,
                           ice_root, stdout) {
@@ -532,13 +545,12 @@ h2o.clusterStatus <- function() {
         "http://www.oracle.com/technetwork/java/javase/downloads/jdk8-downloads-2133151.html")
       }
     )
-
-  if(any(grepl("GNU libgcj", jver))) {
-    stop("Sorry, GNU Java is not supported for H2O.\n",
-         "Please download the latest Java SE JDK 8 from the following URL:\n",
-         "http://www.oracle.com/technetwork/java/javase/downloads/jdk8-downloads-2133151.html")
+  jver_error <- .h2o.check_java_version(jver);
+  if (!is.null(jver_error)) {
+    stop(jver_error, "\n",
+    "Please download the latest Java SE JDK 8 from the following URL:\n",
+    "http://www.oracle.com/technetwork/java/javase/downloads/jdk8-downloads-2133151.html")
   }
-
   if(any(grepl("Client VM", jver))) {
     warning("You have a 32-bit version of Java. H2O works best with 64-bit Java.\n",
             "Please download the latest Java SE JDK 8 from the following URL:\n",
@@ -564,11 +576,13 @@ h2o.clusterStatus <- function() {
   ltrs <- paste0(sample(letters,3, replace = TRUE), collapse="")
   nums <- paste0(sample(0:9, 3,  replace = TRUE),     collapse="")
   name <- paste0("H2O_started_from_R_", gsub("\\s", "_", Sys.info()["user"]),"_",ltrs,nums)
+  args <- c(args, "-Dsys.ai.h2o.debug.allowJavaVersions=11")
   if(enable_assertions) args <- c(args, "-ea")
   class_path <- paste0(c(jar_file, extra_classpath), collapse=.Platform$path.sep)
   args <- c(args, "-cp", class_path, "water.H2OApp")
   args <- c(args, "-name", name)
   args <- c(args, "-ip", ip)
+  args <- c(args, "-web_ip", ip)
   args <- c(args, "-port", port)
   args <- c(args, "-ice_root", slashes_fixed_ice_root)
   if(nthreads > 0L) args <- c(args, "-nthreads", nthreads)
